{*****************************************************************************}
{ [TCDEVENTS]                                                                 }
{   Special Units = DBT.Pas                                                   }
{   Version = 1.43                                                           }
{ [SOURCE]                                                                    }
{   Copyright  1998 by Tom Deprez                                            }
{   (Just to prevent to get multiple components, which all have great         }
{    things but would be a GREAT component with a lot (all) of GREAT things   }
{    That way everybody can benefit of such a GREAT component)                }
{ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ }
{ [COMPONENT]                                                                 }
{ [properties]                                                                }
{   AutoRun : Disable/enable CD autorun feature                               }
{   ID : contains handle of CD-player                                         }
{   Enabled : Disable/enable CD notification                                  }
{   SerialNumber : resturns serial number of CD                               }
{   UPCNumber : returns UPC number of CD                                      }
{   ProductInfo : returns product info of CD                                  }
{   Drives : contains all CD drives on personal computer                      }
{                                                                             }
{ [events]                                                                    }
{   AfterArrival : Fires when new CD is inserted by user                      }
{   AfterRemove  : Fires when CD is removed by user                           }
{                                                                             }
{ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ }
{ [AUTHOR]                                                                    }
{   Author name   = ZifNab (Tom Deprez)                                       }
{   Author e-mail = tom.deprez@uz.kuleuven.ac.be                              }
{ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ }
{ [HISTORY]                                                                   }
{   18/01/1998 : first release                                                }
{   04/02/1998 : changed the order of checking the message (WMDeviceChange)   }
{   21/02/1998 : added disable/enable CD autorun feature                      }
{                added eject/close CD-door                                    }
{   12/03/1998 : fixed a bug (Access violation) 10x to Jaco Vis               }
{   14/03/1998 : changed the new procedures (Open -& CloseDevice) to an       }
{                an appropriate way. 10x for pointing me out, Rav!            }
{   15/05/1998 : Added functions IsAudioCD & GetVolumeName & GetCDInfo        }
{   10/06/1998 : NoDriveTypeAutoRun error on some pc fixed by Matjaz          }
{                Enabled property proposed and implemented by Bartosz Antosik }
{   20/06/1998 : Changed the GetCDInfo to three properties                    }
{   25/10/1998 : Changed GetVolumeName return string (Yoko Maroshi)           }
{                Added procedure GetAllCDDrives, property Drives              }
{                CDevents uses now DBT.pas file (total translation of DBT.H)  }
{   11/11/1998 : Added procedures GetXCDDriveLetter, GetFirstCDDriveLetter    }
{   16/11/1998 : Added pocedure ResetCDOptions again                          }
{   09/12/1998 : Removed a nasty bug! initialised fNotifyEvent! (M. Bostic)   }
{   21/12/1998 : D4 compatibility! (prob. when regis.) (Rob van Oostenrijk)   }
{                                                                             }
{*****************************************************************************}

{*****************************************************************************}
{  [COMMENTS]                                                                 }
{   I very often visit the Delphi page of the www.Experts-Exchange.com site   }
{   It's a place where you can ask or answer questions. One day somebody      }
{   needed to know when a CD is inserted in or ejected out of the CD-Rom      }
{   drive. I found it an interesting question, especially because I could     }
{   not find some 'delphi' source for this problem.                           }
{   I started to search for a solution.                                       }
{   Well, that was the beginning of this component. Where it ends?            }
{   That I don't know, it really depends on the reaction of you! If I get     }
{   some reaction, I'll futher improve it, otherwise it sticks with this.     }
{   Too bad, because I think it can become a great tool.                      }
{                                                                             }
{  [IMPORTANT]                                                                }
{    Made some improvements? : please let me know of, so I can update it      }
{    Need some improvements? : please, just ask and I'll try to make it       }
{    Have you got some ideas? : please, send them to me                       }
{    You use this component? : please, send me an e-mail, why you use this    }
{                              component. An E-mail isn't asked too much.     }
{                              Isn't it? At least let me know of the fact     }
{                              you're using it. It makes me very HAPPY!       }
{  [THANKS]                                                                   }
{    Matvey who brought the first idea to this component                      }
{    Look at history !!!                                                      }
{    Perhaps YOU?                                                             }
{                                                                             }
{ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ }
{  [COPYRIGHT]                                                                }
{    This file is distributed as freeware and without warranties of any       }
{    kind. You can use it in your own applications at your own risk.          }
{                                                                             }
{  [NOTE]                                                                     }
{    It's freeware, but don't hesitate to send me some money if you've        }
{    become rich with the help of this component ;-)                          }
{    It's freeware, but don't hesitate to send me the program or the          }
{    component, where you use this component in. I would be ever gratefull    }
{    to you, thanks.                                                          }
{*****************************************************************************}

unit CDEvents;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DBT, DsgnIntf, Registry, MMSystem;

const cVersion = '1.42';
      cIsAudioCDString = 'AUDIO CD';
      cNoCDDrive = '[None]';
      MCI_INFO_PRODUCT        = $00000100;
      MCI_INFO_FILE           = $00000200;
      MCI_INFO_MEDIA_UPC      = $00000400;
      MCI_INFO_MEDIA_IDENTITY = $00000800;
      MCI_INFO_NAME           = $00001000;
      MCI_INFO_COPYRIGHT      = $00002000;

type
 TNotifyMode = (nmNone, nmEvent, nmScan);

 EMCIDeviceError = class(Exception);

 TDeviceChangeEvent = procedure (Sender : TObject; FirstDriveLetter : char)
                           of object;


 TAboutZifNabProperty = class(TPropertyEditor)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
  end;

 TCDOptions = class(TPersistent)
  private
    fOldAutoRun : Boolean;
    fReset : Boolean;
  protected
    procedure SetCDAutoRun(vAutoRun : Boolean);
    function GetCDAutoRun : Boolean;
  public
    constructor Create;
    destructor Destroy; override;

    procedure ResetCDOptions;
  published
    property Reset: Boolean read fReset write fReset default true;
    property AutoRun: Boolean read GetCDAutoRun write SetCDAutoRun;
 end;

 TCDEvents = class(TComponent)
  private
    { Private declarations }
     FAbout : TAboutZifNabProperty;
     FEnabled : Boolean;
     FFlags : Longint;
     FID : Word;
     fErrCode : longint;
     FWindowHandle: HWND;
     fOptions : TCDOptions;
     fNotifyMode : TNotifyMode;
     fAfterArrival: TDeviceChangeEvent;
     fAfterRemove: TDeviceChangeEvent;
     fCDDrives : TStrings;
     procedure WndProc(var Msg: TMessage);
  protected
   { Protected declarations }
    function GetFirstDriveLetter(unitmask:longint):char;
    procedure WMDeviceChange(var Msg : TWMDeviceChange); dynamic;
    procedure SetOptions(value : TCDOptions);
    procedure SetDevice(aID : word);
    procedure SetEnabled(value : Boolean);
    procedure SetNotifyMode(value : TNotifyMode);
    function GetDevice : word;
    function GetCDInfo(vWord:Word):String;
    function GetSerialNumber: String;
    function GetUPCNumber: String;
    function GetProductInfo: String;
    procedure GetAllCDDrives;
  public
   { Public declarations }
    property SerialNumber: string read GetSerialNumber;
    property UPCNumber: string read GetUPCNumber;
    property ProductInfo: string read GetProductInfo;
    property Drives : TStrings read fCDDrives;
    property ID: Word read GetDevice write SetDevice;

    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure OpenDoor;
    procedure CloseDoor;
    function GetVolumeName(vDrive:char): String;
    function IsAudioCD(vDrive:char):Boolean;
    function GetXCDDriveLetter(vCDNumber: integer): Shortstring;
    function GetFirstCDDriveLetter: ShortString;

   published
   { Published declarations }
    property About : TAboutZifNabProperty read FAbout write FAbout;
    property AfterArrival : TDeviceChangeEvent read fAfterArrival write fAfterArrival;
    property AfterRemove: TDeviceChangeEvent read fAfterRemove write fAfterRemove;
    property Options : TCDOptions read fOptions write SetOptions;
    property Enabled : Boolean read FEnabled write SetEnabled default True;
    property NotifyMode: TNotifyMode read fNotifyMode write SetNotifyMode default nmEvent;
  end;

// SOURCECHANGE
procedure Register; // Added to the component source

implementation

{ Also added to the component source }
procedure Register;
begin
 RegisterComponents('Samples', [TCDEvents]);
end;

{*********************** tAboutZifNabproperty component ***********************}

procedure TAboutZifNabProperty.Edit;
begin
  Application.MessageBox ('        tCDEvents v'+cVersion
                         +#13#10'This component is freeware.'
                         +#13#10'      1998 ZifNab         '
                         +#13#10''
                         +#13#10'mailto:Tom.Deprez@village.uunet.be',
                         'About',
                         MB_OK+ MB_ICONINFORMATION);
end;

function TAboutZifNabProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paMultiSelect, paDialog, paReadOnly];
end;

function TAboutZifNabProperty.GetValue: string;
begin
  Result := '(about)';
end;

(* TCDOptions --------------------------------------------------------------- *)

constructor TCDOptions.Create;
begin
  fReset := True;
  fOldAutoRun := GetCDAutoRun;
end;

destructor TCDOptions.Destroy;
begin
  if fReset then ResetCDOptions;
  inherited Destroy;
end;


procedure TCDOptions.SetCDAutoRun(vAutoRun : Boolean);
 (* this code comes from Delphi Developer Support *)
var
 reg: TRegistry;
 AutoRunSetting : integer;
begin
 reg := TRegistry.Create;
 try
  with reg do begin
   RootKey := HKEY_CURRENT_USER;
   LazyWrite := false;
   if KeyExists('Software\Microsoft\Windows\CurrentVersion\Policies\Explorer') then
    if OpenKey('Software\Microsoft\Windows\CurrentVersion\Policies\Explorer',
           false) then
     begin
      try
       case GetDataType('NoDriveTypeAutoRun') of
        rdInteger : AutoRunSetting:= ReadInteger('NoDriveTypeAutoRun');
        rdBinary  : ReadBinaryData('NoDriveTypeAutoRun',
                                  AutoRunSetting,
                                  SizeOf(AutoRunSetting));
       end;
       if vAutoRun then AutoRunSetting := AutoRunSetting and not (1 shl 5)
        else AutoRunSetting := AutoRunSetting or (1 shl 5);
       case GetDataType('NoDriveTypeAutoRun') of
        rdInteger : WriteInteger('NoDriveTypeAutoRun', AutoRunSetting);
        rdBinary  : WriteBinaryData('NoDriveTypeAutoRun',
                                  AutoRunSetting,
                                  SizeOf(AutoRunSetting));
       end;
      finally
       CloseKey;
      end;
     end;
  end;
 finally
  Reg.Free;
 end;
end;

function TCDOptions.GetCDAutoRun : Boolean;
 (* this code comes from Delphi Developer Support *)
var reg : TRegistry;
    AutoRunSetting : integer;
begin
 reg := TRegistry.Create;
 try
  with reg do begin
   Rootkey := HKEY_CURRENT_USER;
   if KeyExists('Software\Microsoft\Windows\CurrentVersion\Policies\Explorer') then
    if OpenKey('Software\Microsoft\Windows\CurrentVersion\Policies\Explorer', false) then
    begin
     try
      case GetDataType('NoDriveTypeAutoRun') of
       rdInteger : AutoRunSetting:= ReadInteger('NoDriveTypeAutoRun');
       rdBinary  : ReadBinaryData('NoDriveTypeAutoRun',
                                  AutoRunSetting,
                                  SizeOf(AutoRunSetting));
      end;
     finally
      CloseKey;
     end;
    end;
  end;
 finally
  reg.Free;
 end;
 Result := not ((AutoRunSetting and (1 shl 5)) <> 0);
end;

procedure TCDOptions.ResetCDOptions;
begin
 SetCDAutoRun(fOldAutoRun);
end;


(* TCDEvents ---------------------------------------------------------------- *)


function TCDEvents.GetVolumeName(vDrive: Char): String;
var MaximumComponentLength : dWord;
    FileSystemFlags : dWord;
    VolumeName : String;
    i : integer;
begin
 SetLength(VolumeName, 64);
 GetVolumeInformation(PChar(vDrive+':\'),
                      PChar(VolumeName),
                      Length(VolumeName),
                      nil,
                      MaximumComponentLength,
                      FileSystemFlags,
                      nil,
                      0);
 Result := Copy(VolumeName, 0, StrLen(PChar(VolumeName)));
end;

function TCDEvents.IsAudioCD(vDrive: Char) : Boolean;
var VolumeName : String;
    i : integer;

begin
 Result := False;
 VolumeName:= GetVolumeName(vDrive);
 for i := 1 to Length(VolumeName) do
   VolumeName[i] := UpCase(VolumeName[i]);
  if lStrCmp(PChar(VolumeName), cIsAudioCDString)=0 then result := true;
end;

function TCDEvents.GetCDInfo(vWord:Word):String;
var msp : TMCI_INFO_PARMS;
    MediaString : array[0..255] of char;
    ret : longint;
begin
 FillChar(MediaString, SizeOf(MediaString), #0);
 FillChar(msp, sizeof(msp), #0);
 msp.lpstrReturn := @MediaString;
 msp.dwRetSize := 255;
 ret := mciSendCommand(ID,
                       MCI_INFO,
                       vWord,
                       longint(@msp));
 if Ret <> 0 then begin
   MciGetErrorString(ret, @MediaString, SizeOf(MediaString));
   Result := StrPas(MediaString);
 end else Result := StrPas(MediaString);
end;

function TCDEvents.GetSerialNumber: string;
begin
 Result := GetCDInfo(MCI_INFO_MEDIA_IDENTITY);
end;

function TCDEvents.GetUPCNumber: string;
begin
 Result := GetCDInfo(MCI_INFO_MEDIA_UPC);
end;

function TCDEvents.GetProductInfo: string;
begin
 Result := GetCDInfo(MCI_INFO_Product);
end;

procedure TCDEvents.GetAllCDDrives;
var
  Buffer : array[0..500] of char;
  TmpPC  : PChar;

begin
  GetLogicalDriveStrings(SizeOf(Buffer),Buffer);
  TmpPC := Buffer;
  fCDDrives.Clear;
  fCDDrives.BeginUpdate;
  try
   while TmpPC[0] <> #0 do begin
    if GetDriveType(TmpPC) = Drive_CDROM then fCDDrives.Add(TmpPC);
    TmpPC := StrEnd(TmpPC)+1;
   end;
  finally
   fCDDrives.EndUpdate;
  end;
 end;

function TCDEvents.GetXCDDriveLetter(vCDNumber: integer):Shortstring;
begin
 Result := cNoCDDrive;
 if (fCDDrives.Count > -1) and (fCDDrives.Count >= vCDNumber) then begin
  Result := fCDDrives[vCDNumber];
 end;
end;

function TCDEvents.GetFirstCDDriveLetter: Shortstring;
begin
 Result := GetXCDDriveLetter(0);
end;

constructor TCDEvents.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FWindowHandle := AllocateHWnd(WndProc);
  fOptions := TCDOptions.Create;
  fEnabled := True;
  fNotifyMode := nmEvent;
  fID := 0;
  fCDDrives := TStringList.Create;
  GetAllCDDrives;
end;

destructor TCDEvents.Destroy;
begin
  fCDDrives.Free;
  ID := 0;
  fOptions.Free;
  DeallocateHWnd(FWindowHandle);
  inherited Destroy;
end;

procedure TCDEvents.SetOptions(value : TCDOptions);
begin
 fOptions.Assign(value);
end;

procedure TCDEvents.SetNotifyMode(value : TNotifyMode);
begin
 if fNotifyMode <> value then
  fNotifyMode := value;
end;

procedure TCDEvents.WndProc(var Msg: TMessage);
begin
     if (Msg.Msg = WM_DEVICECHANGE) then begin
      if FEnabled and (fNotifyMode = nmEvent) then
       try
         WMDeviceChange(TWMDeviceChange(Msg));
       except
         Application.HandleException(Self);
       end
     end
    else
      Msg.Result := DefWindowProc(FWindowHandle, Msg.Msg, Msg.wParam, Msg.lParam);
end;

function TCDEvents.GetFirstDriveLetter(unitmask : longint):char;
var DriveLetter : shortint;
begin
 DriveLetter := Ord('A');
 while (unitmask and 1)=0  do begin
  unitmask := unitmask shr 1;
  inc(DriveLetter);
 end;
 Result := Char(DriveLetter);
end;

procedure TCDEvents.WMDeviceChange(var Msg : TWMDeviceChange);
var lpdb : PDEV_BROADCAST_HDR;
    lpdbv : PDEV_BROADCAST_VOLUME;
begin
 (* received a wm_devicechange message *)
  lpdb := PDEV_BROADCAST_HDR(Msg.dwData);
  (* look at the event send together with the wm_devicechange message *)
   case Msg.Event of
    DBT_DEVICEARRIVAL : begin
      if lpdb^.dbch_devicetype = DBT_DEVTYP_VOLUME then begin
       lpdbv := PDEV_BROADCAST_VOLUME(Msg.dwData);
       if (lpdbv^.dbcv_flags and DBTF_MEDIA) = 1 then
        if Assigned(fAfterArrival) then
         fAfterArrival(Self,
                       GetFirstDriveLetter(lpdbv^.dbcv_unitmask));
      end;
     end;
    DBT_DEVICEREMOVECOMPLETE : begin
      if lpdb^.dbch_devicetype = DBT_DEVTYP_VOLUME then begin
       lpdbv := PDEV_BROADCAST_VOLUME(Msg.dwData);
       if (lpdbv^.dbcv_flags and DBTF_MEDIA) = 1 then
        if Assigned(fAfterRemove) then
         fAfterRemove(Self,
                      GetFirstDriveLetter(lpdbv^.dbcv_unitmask));
      end;
     end;
   end;
 end;

procedure TCDEvents.SetEnabled(value: Boolean);
begin
 if (FEnabled <> value) then begin
  FEnabled := Value;
 end;
end;

{~~~ MCI Commands ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~}

(* very rudimentary implementation !!! *)

function TCDEvents.GetDevice : word;
var OpenParms : TMCI_Open_Parms;
begin
 if not (csDesigning in ComponentState) then begin
  if fID=0 then begin
   FFlags := 0;
   FFlags := mci_notify or mci_open_type or mci_open_shareable;
   OpenParms.lpstrDeviceType := 'CDAudio';
   OpenParms.dwCallback := 0;
   fErrCode := mciSendCommand(0, mci_open, FFlags, Longint(@OpenParms));
   if FErrCode <> 0 then {problem opening device}
      raise EMCIDeviceError.Create('error')
    else {device successfully opened}
     begin
      fID := OpenParms.wDeviceID;
     end;
  end;
  Result := fID;
 end;
end;

procedure TCDEvents.SetDevice(aID:Word);
var
  GenParms: TMCI_Generic_Parms;
begin
 if not (csDesigning in ComponentState) then begin
  if (aID=0) and (fID <>0) then begin
   FFlags := 0;
   FFlags := mci_notify or mci_open_type or mci_open_shareable; //was mci_notify
   FErrCode := mciSendCommand(fID, mci_Close, FFlags, Longint(@GenParms));
   if FErrCode = 0 then
     begin
       fID := 0;
     end
  end
 end;
end;

procedure TCDEvents.OpenDoor;
var
  SetParms: TMCI_Set_Parms;
begin
  FFlags := 0;
  FFlags := mci_notify or mci_set_door_open;
  SetParms.dwCallback := 0;
  fErrCode := mciSendCommand(ID, mci_Set, FFlags, Longint(@SetParms));
end;

procedure TCDEvents.CloseDoor;
var
  SetParm: TMCI_Set_Parms;
begin
  FFlags := 0;
  FFlags := mci_notify or mci_set_door_closed;
  SetParm.dwCallback := 0;
  fErrCode := mciSendCommand( ID, mci_Set, FFlags, Longint(@SetParm) );
end;

end.
